#ifndef TIMESTAMP_HH
#define TIMESTAMP_HH

#include "Time.hh"
#include <iostream>

//======================================  TimeStamp Class
/**  Maintain a printable time-stamp
  *  @author John Zweizig
  *  @version $Revision: 1.1 $; Modified $Date: 2002/05/22 13:41:20 $
  */
class TimeStamp {
public:
  /**  Construct a TimeStamp object and initialize it to the current
    *  time.
    *  @memo Default constructor
    */
  TimeStamp(void);

  /**  Destroy a timeStamp object.
    *  @memo Destructor
    */
  ~TimeStamp(void);

  /**  Set the timestamp time to the current wall time.
    *  @memo Set the time to now.
    */
  TimeStamp& set(void);

  /**  Set the timestamp to the specified time.
    *  @memo Set the time.
    *  @param t Time to which the time stamp is to be set.
    */
  TimeStamp& set(const Time& t);

  /**  Get the time as a printable string in the standard yyyy.m.dd-hh.mm.ss 
    *  form.
    *  @memo Get a printable time string.
    *  @return Pointer to ta current time string.
    */
  const char* getString() const;

  /**  Get most recent set time.
    */
  Time getTime(void) const;

private:
  Time mT0;
  mutable bool mValid;
  mutable char mStamp[32];
};

inline Time 
TimeStamp::getTime(void) const {
    return mT0;
}

inline std::ostream& 
operator<<(std::ostream& out, const TimeStamp& t) {
    return out << t.getString();
}


#endif  // TIMESTAMP_HH
