#ifndef STR_STACK_HH
#define STR_STACK_HH

#include "eval_stack.hh"
#include <string>
#include <vector>


class str_token : public eval_token {
public:
    /**  Data types. For now the only data types are string, numeric and 
      *  logical (boolean values). In the future this should be templated 
      *  to a more descriptive class.
      */
    enum type_enum {
      kNone,
      kString,
      kNumeric,
      kLogical
    };
    using eval_token::datype;

public:
    //----------------------------------  Constructors
    str_token(void);
    str_token(const std::string& s, type_enum ty=kNone);
    virtual ~str_token(void);

    //----------------------------------  eval_token virtual methods
    str_token* clone(void) const;
    datype  getType(void) const;
    bool    getLogical(void) const;
    double  getNumeric(void) const;
    std::string  getString(void) const;
    void setLogical(bool yorn);
    void setNumeric(double val);
    void setString(const std::string& s);

    //----------------------------------  Specialized methods.
    std::string& refString(void);
    const std::string& refString(void) const;
    void setType(void);

private:
    std::string mValue;
    double      mNumeric;
    type_enum   mType;
};

//====================================== inline functions
inline std::string
str_token::getString(void) const {
    return mValue;
}

inline std::string& 
str_token::refString(void) {
    return mValue;
}

inline const std::string& 
str_token::refString(void) const {
    return mValue;
}

/**  The evaluation stack holds currently active intermediate results.
  *  In the initial implementation, data are stored as strings.
  *  @memo Evaluation stack.
  *  @author John Zweizig
  *  @version 1.0; Last modified July 6, 2004
  */
class str_stack : public eval_stack {
public:
    using eval_stack::push;

    /**  Evaluation stack constructor.
      */
    str_stack(void);

    /** Evaluation stacke destructor. 
      */
    ~str_stack(void) {}

    /** Dump stack contents.
      */
    void dump(std::ostream& out) const;

    /** perform an operation on the top stack entries.
     */
    void evaluate(OpsEnum op);

    /**  Test whether stack entry is numeric.
     */
    bool numeric(int lvl) const;

    /** Push a string on the top of the stack.
     */
    void push(const std::string& str);

    /**  Push a numeric value. The double precision float value is converted
      *  to a string before being pushed on to the stack.
      */
    void push(double val);

    /**  Push a boolean value onto the stack.
      */
    void push(bool val);
};

inline bool 
str_stack::numeric(int off) const {
    return type(off) == str_token::kNumeric;
}

#endif // !defined(STR_STACK_HH)
