void 
ABPlot(const TSeries& a, const TSeries& b, const char* opts=0) {
    //----------------------------------  Formatting parameters
    float ymargin = 0.1;  // Fractional margin in y.

    //----------------------------------  Get the TSeries parameters.
    int nbin = a.getNSample();
    float stime = a.getStartTime().totalS();
    if (stime > 1000000.0) stime = 0;
    float step  = a.getTStep().GetSecs();

    //----------------------------------  Get the data and real length.
    float* DVecA = new float[nbin];
    int nwd = a.getData(nbin,DVecA);
    float* DVecB = new float[nbin];
    int nwd = b.getData(nbin,DVecB);
    float dtime = stime + step * nwd;

    //----------------------------------  Create and fill the histogram.
    TH1F hista("TS001a", a.getName(), nwd, stime, dtime);
    TH1F histb("TS001b", b.getName(), nwd, stime, dtime);
    float ymin = DVecA[0];
    float ymax = DVecA[0];
    for (int i=0 ; i<nwd ; i++) {
        hista.SetBinContent(i+1, DVecA[i]);
	if (DVecA[i] < ymin) ymin = DVecA[i];
	if (DVecA[i] > ymax) ymax = DVecA[i];
        histb.SetBinContent(i+1, DVecB[i]);
    }
    delete[] DVecA;
    delete[] DVecB;
    float yavg = 0.5 * (ymin + ymax);
    float ydif = yavg - ymin;
    ymin = yavg - (1.0 + ymargin)*ydif;
    ymax = yavg + (1.0 + ymargin)*ydif;
    if (ymin < 0.0 && yavg-ydif >= 0.0) ymin = 0.0;
    hista.SetMinimum(ymin);
    hista.SetMaximum(ymax);

    //----------------------------------  Set the plotting options
    gStyle->SetOptStat(0);
    if (gROOT->GetSelectedPad()) (gROOT->GetSelectedPad()->GetCanvas())->cd();
    else                         new TCanvas("cGDS", "GDS Canvas");
    if (gROOT->FindObject("padT")) delete gROOT->FindObject("padT");
    TPad* padT = new TPad("padT", "Time Series", 0.0, 0.0, 1.0, 1.0);
    padT->Draw();
    padT->cd();
    padT->SetBottomMargin(0.12);
    padT->SetLeftMargin(0.15);
    padT->SetTicks(1,1);
    hista.SetLineColor(kRed);
    hista.DrawCopy(opts);
    histb.DrawCopy("SAME");

    //---------------------------------- Label axes
    TText l1; 
    l1.SetTextAlign(22);
    l1.SetTextSize(0.05);
    l1.SetTextAngle(90.);
    l1.DrawTextNDC(0.03, 0.50, "ADC Signal (adc units)");

    l1.SetTextAngle(0.);
    l1.DrawTextNDC(0.50, 0.03, "Time (seconds)");

    padT->GetCanvas()->Update();
}
