#ifndef LMSG_TRANSOUTPUT_HH
#define LMSG_TRANSOUTPUT_HH

#include "lmsg/MsgTypes.hh"
#include <time.h>
#include <string>
#include <stdexcept>

#if __SUNPRO_CC == 0x500
#define NOMEMTEMPLATE
#endif

class Time;

namespace lmsg {

  class Buffer;

  class TransOutput {
  public:
    TransOutput(Buffer& b);
    TransOutput(void);
    void setBuffer(Buffer& b);
    size_type write(const char*   x, size_type N=1) throw(std::runtime_error);
    size_type write(const std::string* x, size_type N=1) throw(std::runtime_error);
    size_type write(const short*  x, size_type N=1) throw(std::runtime_error);
    size_type write(const int*    x, size_type N=1) throw(std::runtime_error);
    size_type write(const long*   x, size_type N=1) throw(std::runtime_error);
    size_type write(const float*  x, size_type N=1) throw(std::runtime_error);
    size_type write(const double* x, size_type N=1) throw(std::runtime_error);
    size_type write(const Time*   x, size_type N=1) throw(std::runtime_error);
    TransOutput& operator<<(const char* x) throw(std::runtime_error);
#ifdef NOMEMTEMPLATE
    TransOutput& operator<<(const char& x) throw(std::runtime_error);
    TransOutput& operator<<(const std::string& x) throw(std::runtime_error);
    TransOutput& operator<<(const short& x) throw(std::runtime_error);
    TransOutput& operator<<(const int& x) throw(std::runtime_error);
    TransOutput& operator<<(const long& x) throw(std::runtime_error);
    TransOutput& operator<<(const float& x) throw(std::runtime_error);
    TransOutput& operator<<(const double& x) throw(std::runtime_error);
    TransOutput& operator<<(const Time& x) throw(std::runtime_error);
#else
    template<class T> TransOutput& operator<<(const T& x) throw(std::runtime_error);
#endif
    void align(size_type N);
    size_type getNBytes(void) const throw();

  private:
    char*  mData;
    size_type mIndex;
    size_type mLength;
  };  //  class lmsg::TransOutput

}    // namespace lmsg



inline lmsg::size_type
lmsg::TransOutput::getNBytes(void) const throw() {
    return mIndex;
}

//--------------------------------------  Implement << by template;
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const char* x) throw(std::runtime_error) {
    std::string s(x);
    write(&s);
    return *this;
}

#if __SUNPRO_CC == 0x500
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const char& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const std::string& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const short& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const int& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const long& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const float& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const double& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const Time& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
#else
template<class T>
inline lmsg::TransOutput&
lmsg::TransOutput::operator<<(const T& x) throw(std::runtime_error) {
    write(&x);
    return *this;
}
#endif

#endif  //  LMSG_TRANSOUTPUT_HH

