#ifndef LMSG_TRANSLATOR_HH
#define LMSG_TRANSLATOR_HH
#include "lmsg/MsgTypes.hh"

namespace lmsg {
  class Buffer;
  class MsgAddr;
  class Grinder;

  class Translator {
  public:
    Translator(const char* fmt=0);
    ~Translator(void);
    size_type Define(const char* fmt);
    size_type Export(const void* in, size_type length, Buffer& b);
    size_type Import(const Buffer& b, void* out, size_type maxlen, 
		     size_type* len);
    void   Reset(void);
    bool   isHeaderOK(void) const;
    bool   isMultiBuffer(void) const;
    void   setHeaderOK(bool yesno=true);
    void   setMultiBuffer(bool yesno=true);
    void   setKeepNames(bool yesno=true);

  private:
    enum tblop {
      o_AFmt=1, o_BFmt, o_DFmt, o_FFmt, o_IFmt, o_NIFmt, o_NSFmt, o_RFmt,
      o_SFmt, o_UBFmt, o_UIFmt, o_USFmt,
      o_Name, o_RepB, o_RepS, o_RepI, o_RepD,
      o_EndB, o_EndS, o_EndI, o_EndD, o_End, o_LTable};
    size_type fmtcd(const char* name, size_type len);
    size_type syntax(const char* fmt, size_type inx);

  private:
    #define HEADEROKFLAG    1
    #define MULTIBUFFERFLAG 2
    size_type mFlags;
    size_type mOffset;
    size_type mOpCd;
    size_type mLevel;
    #define STACKSIZE 8
    size_type mLvPtr[STACKSIZE];
    size_type mLvRpt[STACKSIZE];
    #define NO_TRANS ((short*)-1)
    short* mFormat;
    bool   mKeepNames;

  private:
    static bool   sTabSet;
    static size_type sTabLength[o_LTable];
    static size_type sTabAlign[o_LTable]; 
  };

} //  namespace lmsg

inline bool
lmsg::Translator::isHeaderOK(void) const {
    return (mFlags & HEADEROKFLAG) != 0;
}

inline void 
lmsg::Translator::setHeaderOK(bool yesno) {
    if (yesno) mFlags |=  HEADEROKFLAG;
    else       mFlags &= ~HEADEROKFLAG;
}

inline bool
lmsg::Translator::isMultiBuffer(void) const {
    return (mFlags & MULTIBUFFERFLAG) != 0;
}

inline void 
lmsg::Translator::setMultiBuffer(bool yesno) {
    if (yesno) mFlags |=  MULTIBUFFERFLAG;
    else       mFlags &= ~MULTIBUFFERFLAG;
}

inline void 
lmsg::Translator::setKeepNames(bool yesno) {
    mKeepNames = yesno;
}

#endif   // LMSG_TRANSLATOR_HH
