/** PlaneMonConfig class reads a configuration file containing the 
  * input parameters and passes them to the PlaneMon class DMT monitor.
  * Based on RayleighConfig by Patrick J. Sutton.
  */


#ifndef PLANEMONCONFIG_HH
#define PLANEMONCONFIG_HH 


#include <stdio.h>

class PlaneMonConfig {

    public:
    
    	PlaneMonConfig( char *configFilename );
	
	~PlaneMonConfig();
	
	bool PlaneMonConfig::CheckParameters();
	
	double Stride;
	
	int fLow;
	
	int fHigh;
	
	double lowThreshold;
	
	double highThreshold;
	
	int channelNumber;
	
	char **channelNames;
	
    private:

};


//====================================== Constructor
PlaneMonConfig::PlaneMonConfig( char *configFilename) {

    FILE *in;
    if((in=fopen(configFilename,"r"))==NULL) {
    	std::printf("ERROR: cannot open %s. PlaneMon has crashed.",configFilename);
    } else {
    	std::fscanf(in,"%lf",&Stride);
	std::fscanf(in,"%d",&fLow);
	std::fscanf(in,"%d",&fHigh);
	std::fscanf(in,"%lf",&lowThreshold);
	std::fscanf(in,"%lf",&highThreshold);
	std::fscanf(in,"%d",&channelNumber);
	channelNames = new char*[channelNumber];
	for(int i=0; i<channelNumber; i++) {
		channelNames[i] = new char [30];
		std::fscanf(in,"%s",channelNames[i]);
	}
	fclose(in);
    }
}

//====================================== Destructor
PlaneMonConfig::~PlaneMonConfig() {

    for(int i=0; i<channelNumber; i++) {
    	delete [] channelNames[i];
    }
    delete [] channelNames;
}

bool PlaneMonConfig::CheckParameters() {

    bool status = true;
    
    if(fLow>fHigh) {
    	int temp = fLow;
	fLow = fHigh;
	fHigh = temp;
    }
    
    return status;
}




#endif     //  PLANEMONCONFIG_HH  
