#include <math.h>
#include <TidIO.h>
#include <signal.h>
#include <string.h>
#include <errors.h>
//--->  The next line is needed for RH Linux 7.2 (JGZ 3/5/02)
#include <sys/select.h>

//#define LINUX

#ifdef LINUX
#include <sys/time.h>
void endian_swap(char * pdata, int dsize, int nelements);
#endif

TidIO::TidIO(const int p, bool BLOCK) : port(p)
{
  IServer = 1;
  Alive = 1;
  lastTime = time(0);


  signal(SIGPIPE, SIG_IGN); // ignore SIGPIPE signal

  inbuf = (char *)malloc(INPUT_BUFFER);
  nowPos = 0;

  bool sockOK=0;
  unsigned int ntrial = 0;

  while(!sockOK) {

    ntrial++;

    try {

      sock = socket (AF_INET, SOCK_STREAM, 0);

      if (sock == -1) throw TidError ( "can't create socket");

      /* doesn't work
	 // set socket so it doesn't try to reuse socket
	 if(setsockopt(sock, SOL_SOCKET,SO_REUSEADDR,0,0) == -1 ||
	 setsockopt(sock, SOL_SOCKET,SO_LINGER,0,0) == -1)
	 throw TidError ("Can't set option on socket");
      */
      
      int setRUA = 1;
      setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &setRUA, sizeof(int));

      // set socket to non-blocking io
      if(BLOCK) {
	int flags = fcntl(sock, F_GETFL, 0);
	fcntl(sock, F_SETFL, flags | O_NONBLOCK);
      }

      bzero((char*) &sock_name, sizeof(sock_name));
      sock_name.sin_family = AF_INET;
      sock_name.sin_port = htons( port );
      sock_name.sin_addr.s_addr = htonl(INADDR_ANY);

      if ( bind(sock, (struct sockaddr*) &sock_name, sizeof(sock_name)) == -1) {
	char buf[256];
	sprintf(buf,"Can't bind to port %i\n",port);
	throw TidError ( buf);
      }


      if (listen( sock, BACKLOG ) == -1) 
	throw TidError ("socket can't listen for incomming traffic");
      
      client_sock_length = sizeof(sock_name);

      sockOK = 1;
    }
    catch (TidError err) { 
      HandleError(err); 
      usleep(60000000); // sleep one minute
    }

    if(ntrial >= 15)
      throw TidError ( "FATAL: Have been trying to open the socket for 15 minutes without success... Giving up." );
  }
}


TidIO::TidIO(const char* h, int p) : hostname(h), port(p) {

  IServer = 0;
  Alive = 1;
  lastTime = time(0);

  signal(SIGPIPE, SIG_IGN); // ignore SIGPIPE signal

  inbuf = (char *)malloc(INPUT_BUFFER);
  nowPos = 0;

  struct hostent *he = gethostbyname(hostname.c_str());

  if(!he) throw TidError ( string("Bad hostname : ") + hostname);

  ip = get_ip(he);

  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock == -1) throw TidError ( "can't create socket");

  /* doesn't seem to work
  // set socket so it doesn't try to reuse socket
  if(setsockopt(sock, SOL_SOCKET,SO_REUSEADDR,0,0) == -1 ||
     setsockopt(sock, SOL_SOCKET,SO_LINGER,0,0) == -1)
    throw TidError ("Can't set option on socket");
  */

  // set socket to non-blocking io
  int flags = fcntl(sock, F_GETFL, 0);
  fcntl(sock, F_SETFL, flags | O_NONBLOCK);

  memset(&sock_name, 0, sizeof(sock_name));
  sock_name.sin_family = AF_INET;
  sock_name.sin_port = htons( port );

  // This will work only on Solaris 2.8
  //  inet_pton(AF_INET, ip.c_str(), &sock_name.sin_addr.s_addr); // convert address from presentation format to network format

  sock_name.sin_addr.s_addr = inet_addr(ip.c_str());

  int n;
  if((n = connect(sock, (struct sockaddr *) &sock_name, sizeof(sock_name))) < 0) {
    if(errno != EINPROGRESS)
      {
	char buf[512];
	sprintf(buf,"Can't connect to server %s(%s)",hostname.c_str(),ip.c_str());

	cerr<<buf<<endl;
	
	throw TidError ( buf);
      }
  }

  if(n==0) return;

  fd_set rset, wset;
  struct timeval tval;

  FD_ZERO(&rset);
  FD_SET(sock, &rset);

  int nsec = 180; // 3 minutes
  wset = rset;
  tval.tv_sec = nsec;
  tval.tv_usec = 0;

  if( (n = select(sock+1, &rset, &wset, NULL,
		  nsec ? &tval : NULL)) == 0)
    {
      close(sock);
      throw TidError ("Unable to make connection to data server. Timed out.");
    }

  
}


TidIO::~TidIO() {

  if(IServer) {
    list<int>::iterator p;

    // send END to all
    for(p=csock.begin(); p!=csock.end(); ++p) 
      WriteS(p,"END",3);

    // wait 10 seconds
    usleep(10000000);

    // close everything
    struct linger ling;

    ling.l_onoff = 1; // causes RST to be sent on close()
    ling.l_linger = 0;
    setsockopt(sock, SOL_SOCKET, SO_LINGER, &ling, sizeof(ling));
  
  
    for(p=csock.begin(); p!=csock.end(); ++p) {
      setsockopt(*p, SOL_SOCKET, SO_LINGER, &ling, sizeof(ling));
      if(close(*p)==-1) cerr<<"Client socket could not be closed\n";
    }  
  }

  if(!IServer)
    if(close(sock) == -1) cerr<<"Socket could not be closed\n";

}


char* TidIO::get_ip(struct hostent* he)
{
  int i= 0;
  char *cptr;
  struct in_addr addr;
  void *ptr = &he->h_addr_list[i][0];
  bcopy (ptr, (void*)&addr, sizeof(addr));
  cptr = inet_ntoa(addr);
  return cptr;
} 


int TidIO::Listen() {

  // server doesn't block at accept...
  socklen_t tmps = sizeof(sock_name);
  int tmpsock = accept(sock, (struct sockaddr*) &sock_name, &tmps);
  client_sock_length = tmps;

  if (tmpsock == -1) {
    if (errno == EINTR ||
	errno == EWOULDBLOCK ||
	errno == ECONNABORTED ||
	errno == EPROTO) {
      return -1;
    }
    else {
      cerr<<"Error code: "<<errno<<endl;
      throw TidError ( "Error: socket accept problem");
    }
  }

  TidMessage("Adding new connection to a client.");
  csock.push_back(tmpsock);

  return tmpsock;
}


void TidIO::DumpRead() {

  list<TidString*>* dat = Read(sock);

  EventLW* ev;
  list<TidString*>::iterator p;
  for(p=dat->begin(); p!=dat->end();p++) {
    unsigned int tmp;
    ev = DeMarshal(*p, &tmp);
    delete ev;
    delete *p;
  }

  dat->clear();
}


void TidIO::ReadEventList(list<EventLW*>& out,
			  unsigned int *GPSsec) {

  list<TidString*>* dat = Read(sock);

  *GPSsec = 0;

  EventLW* ev;
  list<TidString*>::iterator p;
  for(p=dat->begin(); p!=dat->end();p++) {
    unsigned int GPS;

    ev = DeMarshal(*p, &GPS);
    out.push_back(ev);

    if(GPS > *GPSsec) *GPSsec = GPS;

    delete *p;
  }

  dat->clear();

  delete dat;
}


list<TidString*>* TidIO::Read(int csock_) { // read data and split it into events

  static time_t lastTime = time(0);

  list<TidString*>* out = new list<TidString*>;

  // read into buffer from socket
  char* in = inbuf + nowPos;


    int len = read(csock_, in, INPUT_BUFFER-nowPos);

    //    cout<<len<<endl;

    if(len == 0) {
      if(time(0) - lastTime >= 30) Alive = 0;
    }

    if (len < 0) {
      if(errno == EWOULDBLOCK) return out; 
      else {
	Close();
	if(time(0) - lastTime >= 30) Alive = 0;
	throw TidError ("Socket read error. Socket was closed.");
      }
    }

    if(len>0) {
      Alive = 1;
      lastTime = time(0);
    }

    in += len;
    nowPos += len; 

    /*
    printf("INBUF:\n");
    for(unsigned int i=0;i<nowPos;i++) printf("%i ",inbuf[i]);
    printf("\n");
    */

    if(nowPos > INPUT_BUFFER) throw TidError ("Overflowing input buffer for socket.");
  
  
    
  if(nowPos > 3 + sizeof(unsigned int)) // may need to do some processing...
    {
      if(strstr(inbuf,"END") == inbuf) {
	TidWarning("Recieved END signal from server. Closing socket.");
	Close();
	return out;
      }

      char* foc;
      if((foc = strstr(inbuf,"TiD")) != inbuf) {

	TidWarning("Corrupted data read from socket. Trying to recover. Some data may be lost.");

	if(foc != NULL) { // copy next valid block at beginning of buffer to recover from error
	  memmove(inbuf,foc,nowPos-(foc-inbuf));
	  bzero(foc,nowPos-(foc-inbuf));
	  //	  cout<<"bzeroing1\n";
	  nowPos -= foc - inbuf;
	}
	else {
	  //	  cout<<"bzeroing2\n";
	  bzero(inbuf,nowPos);
	  nowPos = 0;
	  throw TidError ( "Corrupted data recieved read from socket.");
	}
      }
    }
  else return out;
    

  in = inbuf;
  char *tin;
  unsigned int rea = 0, oldrea=0,
    siz;
  int rem = nowPos;

  while(strstr(in,"TiD") == in &&
	rea <= nowPos) {

    tin = in; // ptr to beginning of this event
    oldrea = rea;

    in+=3; rea+=3; rem-=3; // skip three letters

    if(rem <= (int)sizeof(unsigned int)) // not enough left
      siz = 101010101;
    else {
      sread(&siz, in, sizeof(unsigned int)); // read size

#ifdef LINUX
      endian_swap((char *)&siz,sizeof(unsigned int),1);
#endif

      in += sizeof(unsigned int); 
      rea+=sizeof(unsigned int);
      rem -= sizeof(unsigned int);
    }

    if(rem < (int)(siz + 3)) { // not enough left
      // copy rest of it to beginning
      if(tin != inbuf) {
	memmove(inbuf, tin, nowPos - oldrea);
	//	bzero(inbuf + nowPos - oldrea, oldrea);

	nowPos -= oldrea;
      }

      return out; // normal return
    }

    char *s = new char[siz], *foc;
    sread(s, in, siz); // read data

    in += siz; rea+=siz; rem-=siz;

    if((foc = strstr(in,"dIt")) != in) {
     
      if(foc) {
	foc+=3;
	memmove(inbuf,foc,nowPos-(foc-inbuf));
	bzero(foc,nowPos-(foc-inbuf));
	nowPos -= foc - inbuf;
      }
      else {
	bzero(inbuf,INPUT_BUFFER);
	nowPos = 0;
      }

      //      throw TidError ("Corrupted data recieved from server. Some data may be lost.");
      TidWarning("Corrupted data recieved from server. Some data may be lost.");
      return out;
     
    }

    in+=3; rea+=3; rem-=3;
    
    TidString* tmpstr = new TidString;
    tmpstr->append(s,siz);

    out->push_back(tmpstr);

    delete s;

    //    bzero(tin, in-tin);
  }  

  if(strstr(inbuf,"END") == inbuf) {
    TidWarning("Recieved END signal from server. Closing socket.");
    Close();
    return out;
  }


  if(!rea) 
    {
      memmove(inbuf,in,nowPos-rea);
      //      bzero(inbuf + nowPos - rea, rea);
      nowPos -= rea;
    }

  return out;
}



void TidIO::Write(TidString buffer)
{

  //  Listen(); // accept client connection

  list<int>::iterator p;

  //  cout<<csock.size()<<endl;

  for(p=csock.begin(); p!=csock.end(); p++)
  {

  // writes "TiD" size_unsigned_int buffer "dIt"
  WriteS(p,"TiD",3);

  unsigned int len = buffer.size();
  WriteS(p,(char *)&len,sizeof(unsigned int));

  WriteS(p,buffer.c_str(), buffer.size());

  WriteS(p,"dIt",3);

  }
}


void TidIO::Write(TidString buffer, int p)
{

  //  Listen(); // accept client connection


  //  cout<<csock.size()<<endl;


  // writes "TiD" size_unsigned_int buffer "dIt"
  WriteS(p,"TiD",3);

  unsigned int len = buffer.size();
  WriteS(p,(char *)&len,sizeof(unsigned int));

  WriteS(p,buffer.c_str(), buffer.size());

  WriteS(p,"dIt",3);

}


void TidIO::WriteS(int soc, const char* buffer, unsigned int len) {
  
  
  unsigned int total_sent = 0;
  int bytes_sent = 0;

  while (total_sent < len)
  {
    if((bytes_sent = write(soc, buffer + total_sent, len - total_sent)) <= 0) {
      // sometimes get errno == 11 when lots to do

      if(errno == EINTR ||
	 errno == EWOULDBLOCK ||
	 errno == 11) bytes_sent = 0;
      else {
	{
	  close(soc); // try to close socket
	  perror("Error from write");
	  printf("%i\n",errno);
	  throw TidError ("Socket write. Remove faulty socket from list."); // throw error
	}
      }
    }
    else
      total_sent += bytes_sent;
  }
  

  if(total_sent != len)
    throw TidError ("Socket write. Not all data was sent.");

}



void TidIO::WriteS(list<int>::iterator soc, const char* buffer, unsigned int len) {
  
  static unsigned int maxChunk = 4096; // don't send more than this at a time

  unsigned int total_sent = 0, towrite;
  int bytes_sent = 0;

  while (total_sent < len)
  {
    towrite = len - total_sent;

    if(towrite > maxChunk) towrite = maxChunk;

    if((bytes_sent = write(*soc, buffer + total_sent, towrite)) <= 0) {
      // sometimes get errno == 11 when lots to do

      if(errno == EINTR ||
	 errno == EWOULDBLOCK ||
	 errno == 11) bytes_sent = 0;
      else {
	{
	  close(*soc); // try to close socket
	  csock.erase(soc); // remove from list
	  perror("Error from write");
	  printf("%i\n",errno);
	  throw TidError ("Socket write. Remove faulty socket from list."); // throw error
	}
      }
    }
    else
      total_sent += bytes_sent;
  }
  
  if(total_sent != len)
    throw TidError ("Socket write. Not all data was sent.");

}





EventLW* TidIO::DeMarshal(TidString* s, unsigned int *GPS) { 

  //Actually demarshal

  EventLW* ev = new EventLW;

  // reset
  sread(NULL, s, 0);

  // get channel id number
  unsigned int cid;
  sread(&cid, s, sizeof(unsigned int));

#ifdef LINUX
  endian_swap((char *)&cid,sizeof(unsigned int),1);
#endif

  ev->cid = cid;

  char type; 
  sread(&type,s,1);
  ev->type = (int)type;

  unsigned long int tS, tN, dN;
  long dS;
  sread(&tS, s, sizeof(unsigned long int)); 
  sread(&tN, s, sizeof(unsigned long int)); 
  sread(&dS, s, sizeof(long)); 
  sread(&dN, s, sizeof(unsigned long int)); 

#ifdef LINUX
  endian_swap((char*)&tS,sizeof(unsigned long int),1);
  endian_swap((char*)&tN,sizeof(unsigned long int),1);
  endian_swap((char*)&dS,sizeof(long),1);
  endian_swap((char*)&dN,sizeof(unsigned long int),1);
#endif

  *GPS = (unsigned int)floor((double)tS + 1E-9*(double)tN +
			     (double)dS + 1E-9*(double)dN);

  ev->GPSSec = tS;
  ev->GPSNSec = tN;

  // status
  unsigned int si;
  sread(&si,s,sizeof(unsigned int));

#ifdef LINUX
  endian_swap((char*)&si,sizeof(unsigned int),1);
#endif

  if(si > 0) {
    char *stat = new char[si];

    for(unsigned int i=0; i<si; i++)
      sread(stat+i, s, 1);

    delete stat;
  }

  // filters
  sread(&si,s,sizeof(unsigned int));

#ifdef LINUX
  endian_swap((char*)&si,sizeof(unsigned int),1);
#endif

  tfP tfp;

  for(unsigned int i=0; i<si; i++) {
    sread(&(tfp.t), s, sizeof(unsigned int));
    sread(&(tfp.f), s, sizeof(unsigned int));
    sread(&(tfp.P), s, sizeof(float));

#ifdef LINUX
    endian_swap((char*)&(tfp.t),sizeof(unsigned int),1);
    endian_swap((char*)&(tfp.f),sizeof(unsigned int),1);
    endian_swap((char*)&(tfp.P),sizeof(float),1);
#endif

    ev->tfp.push_back(tfp);
  }

  return ev;
}



void sread(void *ptr, TidString* s, size_t siz) {

  static size_t i = 0;
  char* t = (char *)ptr;

  if(siz == 0) {i = 0; return;}

  for(unsigned int j=0; j<siz && i<s->size(); j++, i++)
    t[j] =  (*s)[i];

  //  if(i==s->size()) throw TidError ( "Corrupted data: string too short.");
    
}


void sread(void *ptr, char* s, size_t siz) {

  char* t = (char *)ptr;

  memcpy(t,s,siz);

  //  for(size_t j=0; j<siz; j++)
  //    t[j] =  s[j];
    
}



bool TidIO::ReadTableofContent(string* source,
			       list<string>* chnnames,
			       list<unsigned int>* cid,
			       list<unsigned int>* fs,
			       double *TRes,
			       double *T0) {
 // read table of content

  // read into buffer from socket
  char* in = inbuf;

  int lenn = read(sock, in+nowPos, INPUT_BUFFER-nowPos);

  if (lenn < 0) return 0; //throw TidError ("Socket read error.");

  nowPos += lenn; 

  if(nowPos > INPUT_BUFFER) throw TidError ("Overflowing input buffer for socket.");
  
  
  if(nowPos >= 3) // may need to do some processing...
    {
      if(strstr(inbuf,"END") == inbuf) {
	TidWarning("Recieved END signal from server. Closing socket.");
	Close();
	return 1;
      }


      char* foc;
      if((foc = strstr(inbuf,"TiD")) != inbuf) {
	if(foc != NULL) { // copy next valid block at beginning of buffer to recover from error
	  memmove(inbuf,foc,foc - inbuf);
	  bzero(foc,foc-inbuf);
	  nowPos = 0;
	}
	else nowPos = 0;
	throw TidError ( "Corrupted data recieved read from socket.");
      }      
    }
  else return 0;

  unsigned int siz;
  if(nowPos >= 3 + sizeof(unsigned int)) {
    sread(&siz, in+3, sizeof(unsigned int)); // read size

#ifdef LINUX
    endian_swap((char*)&siz,sizeof(unsigned int),1);
#endif

  }
  else return 0;

  if(nowPos >= 6 + sizeof(unsigned int) + siz) {
    char* foc;
    if((foc = strstr(inbuf + 3 + siz + sizeof(unsigned int) ,"dIt")) != inbuf + 3 + siz + sizeof(unsigned int)) {
	if(foc != NULL) { // copy next valid block at beginning of buffer to recover from error
	  memmove(inbuf,foc,foc - inbuf);
	  bzero(foc,foc-inbuf);
	  nowPos = 0;
	}
	else nowPos = 0;
	throw TidError ( "Corrupted data recieved read from socket.");
    }      
  }
  else return 0;

  // everything is there, read t of c
  
  char buf[2048];
  char *start = in + 3 + sizeof(unsigned int);
  unsigned int here = 0;
  size_t len;
  
  if((len = strcspn(start,"|")) >= nowPos - 3)
    throw TidError ( "Malformed table of content." );

  memcpy(buf,start,len);
  buf[len] = '\x0';

  *source = string(buf);

  //  cout<<"Source: "<<source<<endl;

  start += 1 + len;
  here += 1 + len;

  if((len = strcspn(start,"|")) >= nowPos - 3)
    throw TidError ( "Malformed table of content." );

  memcpy(buf,start,len);
  buf[len] = '\x0';

  *TRes = atof(buf);

  start += 1 + len;
  here += 1 + len;

  if((len = strcspn(start,"|")) >= nowPos - 3)
    throw TidError ( "Malformed table of content." );

  memcpy(buf,start,len);
  buf[len] = '\x0';

  *T0 = atof(buf);

  start += 1 + len;
  here += 1 + len;

  //  cout<<"Channel:\t\tcid:\t\tsampling\n";

  while(here < siz) {
    
    if((len = strcspn(start,"|")) >= nowPos - 3)
      throw TidError ( "Malformed table of content." );
    
    memcpy(buf,start,len);
    buf[len] = '\x0';

    string tmp(buf);
    chnnames->push_back(tmp);

    start += 1 + len;
    here += 1 + len;



    if((len = strcspn(start,"|")) >= nowPos - 3)
      throw TidError ( "Malformed table of content." );
    
    memcpy(buf,start,len);
    buf[len] = '\x0';

    unsigned int tmpi = atoi(buf);
    cid->push_back(tmpi);

    start += 1 + len;
    here += 1 + len;



    if((len = strcspn(start,"|")) >= nowPos - 3)
      throw TidError ( "Malformed table of content." );
    
    memcpy(buf,start,len);
    buf[len] = '\x0';

    unsigned int tmpi2 = atoi(buf);
    fs->push_back(tmpi2);

    start += 1 + len;
    here += 1 + len;

    //    cout<<tmp<<"\t\t"<<tmpi<<"\t\t"<<tmpi2<<endl;
  }

  if(strstr(start,"dIt")!=start) 
    throw TidError ( "Malformed table of content." );
  
  start += 3;
  here += 3;

  here += 7;

  if(here < nowPos) {
    memmove(inbuf,start,nowPos-here);
    //    bzero(inbuf+nowPos-here,here);
    nowPos -= here;
  }
  else {
    //    bzero(inbuf,here);
    nowPos=0;
  }
  return 1;
}


void TidIO::WritePlain(const TidString AA) {
 // write plain string + carriage return

  const char* buffer;

  unsigned int total_sent;
  int bytes_sent;

  list<int>::iterator p;

  TidString A = AA;

  A += "\n";

  for(p=csock.begin(); p!=csock.end(); ++p) {

    total_sent = 0;
    bytes_sent = 0;
    buffer = A.c_str();

    while (total_sent < A.size())
      {
	if((bytes_sent = write(*p, buffer + total_sent, A.size() - total_sent)) <= 0) {
	  // sometimes get errno == 11 when lots to do

	  if(errno == EINTR) bytes_sent = 0;
	  else {
	    if(errno != 11) {
	      close(*p); // try to close socket
	      csock.erase(p); // remove from list
	      perror("Error from write");
	      printf("%i\n",errno);
	      throw TidError ("Socket write. Remove faulty socket from list."); // throw error
	    }
	  }
	}
	else
	  total_sent += bytes_sent;
      }

    if(bytes_sent != (int)A.size())
      throw TidError ("Socket write. Not all data was sent.");
    
  }
}


bool TidIO::ReadPlain(TidString **A) {

  if(csock.size() < 1) return 0;

  // read into buffer from socket
  char* in = inbuf + nowPos;


  int len = read(*(csock.begin()), in, INPUT_BUFFER-nowPos);

  if (len < 0) return 0; //throw TidError ("Socket read error.");

  in += len;
  nowPos += len; 

  if(nowPos > INPUT_BUFFER) throw TidError ("Overflowing input buffer for socket.");
  
  char* en;

  if((en = strchr(inbuf,'\n'))==NULL)
    return 0;

  unsigned int lenstr = en - inbuf - 1;

  if(lenstr)
    *A = new TidString(inbuf, lenstr);

  if(nowPos > lenstr) {
    memmove(inbuf, en + 1, nowPos - lenstr - 1);
    bzero(en + 1, nowPos - lenstr - 1);
  }
  else
    bzero(inbuf,nowPos);

  nowPos = 0;

  if(!lenstr) return 0;

  return 1;
}


void TidIO::HangUp ( int so ) { // close socket

  close(so);
  
  list<int>::iterator p;
  for(p=csock.begin(); p!=csock.end(); ++p) {
    if(*p == so) {csock.erase(p);}
  }  
}


void TidIO::HangUp () { // close socket

  list<int>::iterator p = csock.begin();
  close(*p);
  csock.erase(p);

}

#ifdef LINUX
void endian_swap(char * pdata, int dsize, int nelements)

{

        int i,j,indx;
        char tempbyte;

        if (dsize <= 1) return;

        for (i=0; i<nelements; i++)
        {
                indx = dsize;
                for (j=0; j<dsize/2; j++)
                {
                        tempbyte = pdata[j];
                        indx = indx - 1;
                        pdata[j] = pdata[indx];
                        pdata[indx] = tempbyte;
                }

                pdata = pdata + dsize;
        }

        return;

}
#endif


void TidIO::SocketDump(FILE *out) {

  int len = read(sock, inbuf, INPUT_BUFFER);

  if(len == 0) {
    if(time(0) - lastTime >= 300) Alive = 0;
  }

  if (len < 0) {
    if(time(0) - lastTime >= 300) Alive = 0;

    if(errno == EWOULDBLOCK) return; 
    else {
      cerr<<"Socket read error.\n";
      return;
    }
  }

  if(len) {
    Alive = 1;
    lastTime = time(0);
    fwrite(inbuf, 1, len, out);
  }
}


void TidIO::SocketUnDump(FILE *in) {

  char tmp[4000];
  list<int>::iterator p;
  unsigned int lon;

  for(;;) {

    lon = fread(tmp,1,4000,in);
    if(lon)
      for(p=csock.begin(); p!=csock.end(); p++)
	WriteS(p,tmp,lon);

    if(feof(in)) break;
  }
}

void TidIO::WaitForClient () {

  while(!csock.size()) {

    Listen();

    usleep(100000);

  }
}

void TidIO::Close() {
  close(sock);
}

void TidIO::Write2All(char *dat, unsigned int len) {
  list<int>::iterator p;
  for(p=csock.begin(); p!=csock.end(); p++)
	WriteS(p,dat,len);
}
