#ifndef TRIG_TRIGWRITER_HH
#define TRIG_TRIGWRITER_HH

#include "lmsg/MsgTypes.hh"
#include <string>


namespace trig {

  class Segment;
  class TrigBase;
  class TrigProc;

  /**  The TriggerWriter class is an API for writing triggers to various 
    *  trigger output databases and formats.
    *  @memo Trigger writer API
    *  @author J. Zweizig
    *  @version 1.0; Last Modified September 23, 2005
    */
  class TrigWriter {
  public:
    enum trig_mode {
      kNone, 
      kMgr,
      kWriter,
      kSegWrt
    };

  public:
      TrigWriter(void);
      virtual ~TrigWriter(void);
      virtual lmsg::error_type addTrigger(const TrigBase& t, const TrigProc& p)=0;
      virtual lmsg::error_type addSegment(const Segment& s, const TrigProc& p)=0;
      virtual void clear(void) = 0;
      virtual bool empty(void) const;
      virtual lmsg::index_type getDebug(void) const;
      virtual trig_mode getMode(void) const = 0;
      virtual int getNSegs(void) const;
      virtual int getNTrigs(void) const;
      virtual void setDebug(lmsg::index_type d);
      virtual lmsg::error_type setProcess(const TrigProc& p);
      virtual lmsg::error_type write(const std::string& file) const = 0;
  private:
      lmsg::index_type mDebug;
  };

} // namespace trig

#endif // TRIG_TRIGWRITER_HH
