#include "Time.hh"
#include "TrigBase.hh"
#include "Segment.hh"
#include "TrigClient.hh"
#include "SigFlag.hh"
#include "rndm.hh"
#include <string>
#include <cmath>
#include <unistd.h>
#include <time.h>

#define  PIDCVSHDR "$Header: /ldcg_server/common/repository_gds/gds/Services/TrigMgr/TrigRndm.cc,v 1.5 2005/10/05 19:50:36 jzweizig Exp $"
#define  PIDTITLE  "Random trigger generation utility"
#include "ProcIdent.hh"

using namespace std;

//======================================  TrigRndm class definition
class TrigRndm : public TrigClient {
public:
  TrigRndm(int argc, const char* argv[]);
  ~TrigRndm(void);
  int Application(void);
private:
  int     mGenerate;
  double  mRate;
  std::string mTrigName;
  std::string mIfoName;
  int     mDispMask;
  bool    mSyntax;
  int     mNGener;
  int     mNError;
  double  mDuration;
  bool    mSegment;
  SigFlag mTerm;
};

//======================================  Main function
int
main(int argc, const char* argv[]) {
    TrigRndm app(argc, argv);
    int rc = app.Application();
    return rc;
}

//======================================  TrigRndm constructor (initializer)
TrigRndm::TrigRndm(int argc, const char* argv[]) 
  : TrigClient(trig::TrigWriter::kNone), mGenerate(0), mRate(1.0), 
    mDispMask(trig::d_all), mSyntax(false), mNGener(0), mNError(0), 
    mDuration(1.0), mSegment(false), mTerm(SIGINT)
{
    for (int i=1 ; i<argc ; i++) {
        string argi = argv[i];
	if (argi == "-disp") {
	    mDispMask = strtol(argv[++i], 0, 0);
	} else if (argi == "-debug") {
	    setDebug(strtol(argv[++i], 0, 0));
	} else if (argi == "-dt") {
	    mDuration = strtod(argv[++i], 0);
	} else if (argi == "--help") {
	    cout << "TrigRndm generates random triggers and sends them to the"
		 << "  a file or to the TrigMgr via" << endl;
	    cout << "the TrigClient interface."  << endl;
	} else if (argi == "-ifos") {
	    mIfoName = argv[++i];
	} else if (argi == "-name") {
	    mTrigName = argv[++i];
	} else if (argi == "-ntrig") {
	    mGenerate = strtol(argv[++i], 0, 0);
	} else if (argi == "-rate") {
	    mRate     = strtod(argv[++i], 0);
	} else if (argi == "-segment") {
	    mSegment  = true;
	} else {
	    cout << "Syntax error. Unidentified argument: " << argi << endl;
	    mSyntax = true;
	}
	if (mSyntax) {
	    cout << "TrigRndm command syntax: " << endl;
	    cout << "TrigRndm [-name <name>] [-ntrig <ntrig>] [-rate <rate>] "
		 << "[-disp <disp-mask>] [-dt <delta-t>]" << endl;
	    return;
	}
    }
    mTerm.add(SIGTERM);

    if (getDebug()) cout << "Enrolling trigger client..." << endl;
    int rc = enroll(trig::TrigWriter::kMgr);
    if (getDebug()) cout << "Registration complete, rc=" << rc << endl;
}
 
//======================================  TrigRndm destructor (clean up).
TrigRndm::~TrigRndm(void) {
    if (!mSyntax) {
        cout << "TrigRndm completed. Trigger name: " << mTrigName
	     << " Triggers Generated: " << mNGener << " Total errors "
	     << mNError << endl;
    }
}

//======================================  Random trigger generation
int
TrigRndm::Application(void) {
    if (mSyntax) return 1;
    int rc = 0;
    double delay(0.0);
    while (!mTerm && (!mGenerate || mNGener < mGenerate)) {

        //------------------------------  Wait a random time.
        if (delay != 0.0) {
            long long nsec = (long long)(delay * 1E9);
            timespec wait = {nsec / 1000000000LL, nsec % 1000000000LL};
            nanosleep (&wait, 0);
	    // useconds_t dt = useconds_t(delay * 1000000.0);
	    // usleep(dt);
	}

	Time now = Now();
	int tErr(0);
	if (mSegment) {
	    //--------------------------  Generate and send a segment.
	    trig::Segment s(mTrigName.c_str(), 1, now, now+mDuration);
	    s.setIfos(mIfoName.c_str());
	    tErr = sendSegment(s);
	} else {
	    //--------------------------  Generate a trigger.
	    trig::TrigBase t("TrigRndm", mTrigName.c_str(), now, mDuration);
	    t.setDisposition(mDispMask);
	    tErr = sendTrigger(t);
	}

	//------------------------------  Bump counts - print errors
	mNGener++;
	if (tErr) {
	    cout << "Error " << tErr << " occurred in ";
	    if (mSegment) cout << "sendSegment" << endl;
	    else          cout << "sendTrigger" << endl;
	    mNError++;
	}
	    
	//------------------------------  Generate next random delay
	delay = -log(Rndm())/mRate;
    }
    return rc;
}
